%{
#include <stdio.h>
#include <string.h>

#include "BTree.h"
#include "global.h"

#ifdef __BORLANDC__
#include "syntax_t.h"
#else
#include "syntax.tab.h"
#endif

int yyparse( void );

int parsing_line;

%}

%option noyywrap
%x longid
%x args

%%

<INITIAL>\\([^\\]|\\\\)*\\ 	{ yylval.str = StringBuf::strDup( yytext ); return T_Ident; }
<INITIAL>"\""([^\"])*"\"" 	{ yylval.str = StringBuf::strDup( yytext ); return T_Ident; }

<INITIAL>"*"		{return T_And;}
<INITIAL>"+"		{return T_Or;}
<INITIAL>"'"            {return T_Not;}
<INITIAL>"="            {return '=';}
<INITIAL>"("            {return '(';}
<INITIAL>")"            {return ')';}
<INITIAL>";"            {return ';';}
<INITIAL>","            {return ',';}
<INITIAL>"t"            {return 't';}
<INITIAL>"t-1"          {return T_t_1;}
<INITIAL>".model"	{return T_model;}
<INITIAL>".endmodel"	{return T_endmodel;}
<INITIAL>"TRUE"		{return T_True;}
<INITIAL>"FALSE"	{return T_False;}
<INITIAL>[a-zA-Z_][a-zA-Z0-9_]* { yylval.str = StringBuf::strDup( yytext ); return T_Ident;}
<INITIAL>-?[0-9]+	{ yylval.str = StringBuf::strDup( yytext ); return T_Int; }
<INITIAL>[ \t\x0d]
<INITIAL>[\n]           { parsing_line++; }
<INITIAL>"#".*"\n"	{ yylval.str = StringBuf::strDup( yytext ); parsing_line++; return T_Comment;}

.       {  yylval.ch = yytext[0]; return T_Anything; }

%%

char *modelName = "main";

void printUsage();

main( int argcnt, char **arg )
{

        if( argcnt != 3 ) {
                printUsage();
                return 1;
        }

        yyin = fopen( arg[1], "r" );
        if( yyin == 0 ) {
	        fprintf( stderr, "Can not open file for reading: %s\n", arg[1] );
                return( 1 );
        }

        convOut = fopen( arg[2], "w" );
        if( convOut == 0 ) {
                fprintf( stderr, "Can not open file for reading: %s\n", arg[2] );
                return( 1 );
        }


        parsing_line = 1;
        if( yyparse() != 0 ) return 1;
}


void printUsage()
{
        fprintf( stderr, "Usage: slif2bnf {input file} {output file}\n\n" );

}