%{

int yylex( );
void yyerror( char *s );

#include "global.h"

#include <stdio.h>
#include <stdlib.h>

#ifdef __BORLANDC__
#include <malloc.h>
#endif


#define ALLOCBTREE (BTree*)alloc.get( sizeof( BTree ) )
#define ALLOCSLIST (StringList*)alloc.get( sizeof( StringList ) )

%}
%union {
  char *str;
  BTree *btree;
  StringList *slist;
  char ch;
}

%token <str> T_t_1	// t-1
%token <btree> T_And
%token <btree> T_Or
%token <btree> T_Not
%token <str> T_Ident
%token <str> T_Int
%token <str> T_Comment
%token <str> T_model
%token <str> T_endmodel
%token <str> T_True
%token <str> T_False
%token <ch>  T_Anything
%type <btree> exp
%type <slist> var
%type <slist> index_list
%type <slist> index

%left T_Or
%left T_And
%nonassoc T_Not


%%
document: T_model T_Ident file T_endmodel T_Ident {
}
;
file:	/*empty */
	| items
;

items:	item 
	| items item
;

item:	command
	| comment
	| T_Anything {
		fputc( $1, convOut );
		printStr( "\n" );
	}
;

comment: T_Comment {
	fprintf( convOut, "--%s", $1+1 );

}
;

command: var '=' exp ';' {
	$1->print();
	printStr( " = " );
	$3->print();
	printStr( ";\n" );

	alloc.clear();
//	    fprintf( stderr, "Command in line : %d\n", parsing_line );
	
	}
;

var:	T_Ident {
		StringList *sl = ALLOCSLIST;
		sl->set( $1 );
		$$ = sl;
	}
	| T_Int {
		StringList *sl = ALLOCSLIST;
		sl->set( $1 );
		$$ = sl;
	}
	| T_Ident '(' index_list ')' {
		StringList *sl = ALLOCSLIST;
		sl->set( $1 );
		StringList *sn = ALLOCSLIST;
		sn->set( "(" );
		sl->append( sn );
		sl->append( $3 );
		sn = ALLOCSLIST;
		sn->set( ")" );
		sl->append( sn );		
		$$ = sl;	
	}
;

index_list:	index {
			$$ = $1;
		}
		| index ',' index_list {
			StringList *sl = ALLOCSLIST;
			sl->set( ", " );
			$$ = sl;
			
			$1->append( sl );
			$1->append( $3 );
			$$ = $1;
		}
;

index:		T_t_1 {
			StringList *sl = ALLOCSLIST;
			sl->set( "t-1" );
			$$ = sl;
		}
		| 't' {
			StringList *sl = ALLOCSLIST;
			sl->set( "t" );
			$$ = sl;
		}
		| var {
			$$ = $1;
		}
;



exp:	var {
		BTree *bt = ALLOCBTREE;
		bt->set( (char*)$1 );
		$$ = bt;
	}
	| T_True {
		BTree *bt = ALLOCBTREE;
		bt->set( "1" );
		$$ = bt;
	}
	| T_False {
		BTree *bt = ALLOCBTREE;
		bt->set( "0" );
		$$ = bt;
	}
	| exp T_Not {
		BTree *bt = ALLOCBTREE;
		bt->set( "!", 0, $1 );
		$$ = bt;
	}
	| exp T_And exp {
		BTree *bt =  ALLOCBTREE;
		bt->set( " & ", $1, $3 );
		$$ = bt;
	}
	| exp T_Or exp {
		BTree *bt =  ALLOCBTREE;
		bt->set( " | ", $1, $3 );
		$$ = bt;
	}
	| '(' exp ')' {
		BTree *bt =  ALLOCBTREE;
		bt->set( "(", $2 );
		$$ = bt;
	}
;

%%

extern int parsing_line;
void yyerror( char *s )
{
	fprintf( stderr, "Syntax error in line %d\n", parsing_line );
}


